######################
# Setup rudder agent #
######################
setup_agent() {

  [ -z "${SERVER}" ] && SERVER="rudder"

  # Install via package manager only
  if [ -z "${PM}" ]
  then
    echo "Sorry your System is not *yet* supported !"
    exit 4
  fi

  # The version given is a file or a URL
  if [ -f "${RUDDER_VERSION}" ] || echo "${RUDDER_VERSION}" | grep "^http" > /dev/null || echo "${RUDDER_VERSION}" | grep "^ftp" > /dev/null
  then
    # localinstall
    if [ "${PM}" = "pkg" ] && LANG=C file "${RUDDER_VERSION}" | grep "gzip compressed data" > /dev/null
    then
      cd /tmp
      gzip -d -c "${RUDDER_VERSION}" | tar xf -
      ${PM_LOCAL_INSTALL} /tmp RudderAgent
    elif [ "${PM}" = "pkg" ]
    then
      ${PM_LOCAL_INSTALL} "${RUDDER_VERSION}" RudderAgent
    else
      ${PM_LOCAL_INSTALL} "${RUDDER_VERSION}"
    fi
  else
    # Install
    ${PM_INSTALL} rudder-agent
  fi

  # System specific behavior
  #######

  # TODO rhel5 only
  #${PM_INSTALL} pcre openssl db4-devel

  if [ -n "${SERVER}" ]
  then
    echo "${SERVER}" > /var/rudder/cfengine-community/policy_server.dat
    if is_version_valid "${RUDDER_VERSION}" "[3.0 *]"
    then
      rudder agent inventory
    else
      cf-agent -K -D force_inventory -b doInventory
    fi
  fi

  service_cmd rudder-agent start

}

upgrade_agent() {

  # Upgrade via package manager only
  if [ -z "${PM}" ]
  then
    echo "Sorry your System is not *yet* supported !"
    exit 4
  fi

  # Upgrade
  ${PM_UPGRADE} rudder-agent

}

setup_relay() {

  if is_version_valid "${RUDDER_VERSION}" "[3.0 *]"; then
    ${PM_INSTALL} rudder-server-relay
    rudder agent inventory
  else
    setup_relay_old
  fi

}

upgrade_relay() {

  if is_version_valid "${RUDDER_VERSION}" "[3.0 *]"; then
    # Upgrade via package manager only
    if [ -z "${PM}" ]
    then
      echo "Sorry your System is not *yet* supported !"
      exit 4
    fi

    # If you upgrade from a 2.11 relay you will need to install the package
    # Since it's easier to install then update than checking the installed package
    # we decided to go that way
    ${PM_INSTALL} rudder-server-relay
    ${PM_UPGRADE} rudder-server-relay
  fi
}


setup_relay_old() {

  # Install relay dependencies
  if [ "${PM}" = "apt" ]
  then
    ${PM_INSTALL} apache2 apache2-utils rsyslog
    a2enmod dav dav_fs
    a2dissite default
  elif [ "${PM}" = "yum" ]
  then
    ${PM_INSTALL} httpd httpd-tools rsyslog
  else
    ${PM_INSTALL} apache2 apache2-utils rsyslog
    a2enmod dav dav_fs
  fi

  mkdir -p /opt/rudder/etc /var/log/rudder/apache2 /var/rudder/share

  for i in /var/rudder/inventories/incoming /var/rudder/inventories/accepted-nodes-updates
  do
    mkdir -p ${i}
    chmod -R 1770 ${i}
    for group in apache www-data www; do
      if getent group ${group} > /dev/null
      then
        chown -R root:${group} ${i}
        break
      fi
    done
  done

  for i in /opt/rudder/etc/htpasswd-webdav-initial /opt/rudder/etc/htpasswd-webdav
  do
    /usr/bin/htpasswd -bc ${i} rudder rudder
  done

  touch /opt/rudder/etc/rudder-networks.conf

  if [ "${PM}" = "apt" ]
  then
    CONFFILE=/etc/apache2/sites-enabled/rudder-default
  elif [ "${PM}" = "yum" ]
  then
    CONFFILE=/etc/httpd/conf.d/rudder-default.conf
  else
    CONFFILE=/etc/apache2/vhosts.d/rudder-default.conf
  fi

cat > "${CONFFILE}" <<EOF
<VirtualHost *:80>
        ServerAdmin webmaster@localhost
        # Expose the server UUID through http
        Alias /uuid /opt/rudder/etc/uuid.hive
        <Directory /opt/rudder/etc>
                Order deny,allow
                Allow from all
        </Directory>
        # WebDAV share to receive inventories
        Alias /inventories /var/rudder/inventories/incoming
        <Directory /var/rudder/inventories/incoming>
                DAV on
                AuthName "WebDAV Storage"
                AuthType Basic
                AuthUserFile /opt/rudder/etc/htpasswd-webdav-initial
                Require valid-user
                Order deny,allow
                # This file is automatically generated according to
                # the hosts allowed by rudder.
                Include /opt/rudder/etc/rudder-networks.conf
                <LimitExcept PUT>
                        Order allow,deny
                        Deny from all
                </LimitExcept>
        </Directory>
        # WebDAV share to receive inventories
        Alias /inventory-updates /var/rudder/inventories/accepted-nodes-updates
        <Directory /var/rudder/inventories/accepted-nodes-updates>
                DAV on
                AuthName "WebDAV Storage"
                AuthType Basic
                AuthUserFile /opt/rudder/etc/htpasswd-webdav
                Require valid-user
                Order deny,allow
                # This file is automatically generated according to
                # the hosts allowed by rudder.
                Include /opt/rudder/etc/rudder-networks.conf
                <LimitExcept PUT>
                        Order allow,deny
                        Deny from all
                </LimitExcept>
        </Directory>
        # Logs
        ErrorLog /var/log/rudder/apache2/error.log
        LogLevel warn
        CustomLog /var/log/rudder/apache2/access.log combined

</VirtualHost>
EOF

  if [ "${PM}" = "apt" ]
  then
    service_cmd apache2 restart
  elif [ "${PM}" = "yum" ]
  then
    service_cmd httpd restart
  else
    service_cmd apache2 restart
  fi

  /var/rudder/cfengine-community/bin/cf-agent -K -D force_inventory

}
